@@ -1,13 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-use Object::Pad 0.27;
+use Object::Pad 0.57;
 
-package Tickit::Widget::Entry 0.31;
+package Tickit::Widget::Entry 0.34;
 class Tickit::Widget::Entry
-   extends Tickit::Widget;
+   :isa(Tickit::Widget);
+
+use Carp;
 
 use Tickit::Style;
 Tickit::Window->VERSION( '0.39' ); # expose_after_scroll default on
@@ -177,51 +179,40 @@ Optional. Callback function to invoke when the C<< <Enter> >> key is pressed.
 
 =cut
 
-has $_text;
-has $_pos_ch;
+has $_text          :reader :param = "";
+has $_pos_ch        :reader(position) :param(position) = 0;
 has $_scrolloffs_co = 0;
 has $_overwrite     = 0;
-has %_keybindings;
-has $_on_enter;
+has %_keybindings = (
+   'C-a' => "key_beginning_of_line",
+   'C-e' => "key_end_of_line",
+   'C-k' => "key_delete_line",
+   'C-u' => "key_backward_delete_line",
+   'C-w' => "key_backward_delete_word",
 
-BUILD
+   'M-b' => "key_backward_word",
+   'M-f' => "key_forward_word",
+
+   'Backspace'   => "key_backward_delete_char",
+   'C-Backspace' => "key_backward_delete_word",
+   'Delete'      => "key_forward_delete_char",
+   'C-Delete'    => "key_forward_delete_word",
+   'End'         => "key_end_of_line",
+   'Enter'       => "key_enter_line",
+   'Home'        => "key_beginning_of_line",
+   'Insert'      => "key_overwrite_mode",
+   'Left'        => "key_backward_char",
+   'C-Left'      => "key_backward_word",
+   'Right'       => "key_forward_char",
+   'C-Right'     => "key_forward_word",
+);
+has $_on_enter      :reader :param = undef;
+
+ADJUST
 {
-   my %params = @_;
-
-   $_text = defined $params{text} ? $params{text} : "";
-   $_pos_ch = defined $params{position} ? $params{position} : 0;
-
    my $textlen = length $_text;
    $_pos_ch = $textlen if $_pos_ch > $textlen;
 
-   # TODO: It'd be nice to get this out of Object::Pad but it's a list on a
-   #   hash and thus not const
-   %_keybindings = (
-      'C-a' => "key_beginning_of_line",
-      'C-e' => "key_end_of_line",
-      'C-k' => "key_delete_line",
-      'C-u' => "key_backward_delete_line",
-      'C-w' => "key_backward_delete_word",
-
-      'M-b' => "key_backward_word",
-      'M-f' => "key_forward_word",
-
-      'Backspace'   => "key_backward_delete_char",
-      'C-Backspace' => "key_backward_delete_word",
-      'Delete'      => "key_forward_delete_char",
-      'C-Delete'    => "key_forward_delete_word",
-      'End'         => "key_end_of_line",
-      'Enter'       => "key_enter_line",
-      'Home'        => "key_beginning_of_line",
-      'Insert'      => "key_overwrite_mode",
-      'Left'        => "key_backward_char",
-      'C-Left'      => "key_backward_word",
-      'Right'       => "key_forward_char",
-      'C-Right'     => "key_forward_word",
-   );
-
-   $self->set_on_enter( $params{on_enter} ) if defined $params{on_enter};
-
    # Since we take keyboard input we almost certainly want to take focus here
    $self->take_focus;
 }
@@ -336,10 +327,10 @@ method reposition_cursor
 {
    my ( $pos_ch ) = @_;
 
-   my $win = $self->window or return;
-
    $_pos_ch = $pos_ch if defined $pos_ch;
 
+   my $win = $self->window or return;
+
    my $new_scrolloffs = $self->_recalculate_scroll( $_pos_ch );
    if( defined $new_scrolloffs ) {
       $_scrolloffs_co = $new_scrolloffs;
@@ -469,7 +460,7 @@ action is invoked; usually bound to the C<Enter> key.
 
 =cut
 
-method on_enter { $_on_enter }
+# generated accessor
 
 method set_on_enter
 {
@@ -484,7 +475,7 @@ Returns the current entry position, in terms of characters within the text.
 
 =cut
 
-method position { $_pos_ch }
+# generated accessor
 
 =head2 set_position
 
@@ -539,6 +530,42 @@ method bind_keys
    }
 }
 
+=head2 make_popup_at_cursor
+
+   $win = $entry->make_popup_at_cursor( $top_offset, $left_offset, $lines, $cols )
+
+I<Since version 0.33.>
+
+Creates a new popup window, as if calling L<Tickit::Window/make_popup> on the
+widget's main window, but with an offset relative to the current cursor
+position.
+
+An offet of (0, 0) will position the popup window's top left corner exactly over
+the cursor; this is likely not what you want.
+
+To position the popup just below the widget, use a top offset of +1:
+
+   $win = $entry->make_popup_at_cursor( +1, 0, $lines, $cols )
+
+To position the popup just above the widget, use a top offset of negative the
+number of lines:
+
+   $win = $entry->make_popup_at_cursor( -$lines, 0, $lines, $cols )
+
+=cut
+
+method make_popup_at_cursor
+{
+   my ( $topoff, $leftoff, $lines, $cols ) = @_;
+
+   $self->window or
+      croak "Cannot ->make_popup_at_cursor on an Entry widget with no window";
+
+   my $pos_x = $self->char2col( $_pos_ch ) - $_scrolloffs_co;
+
+   return $self->window->make_popup( $topoff, $pos_x + $leftoff, $lines, $cols );
+}
+
 =head1 TEXT MODEL METHODS
 
 These methods operate on the text input buffer directly, updating the stored
@@ -555,7 +582,7 @@ Returns the currently entered text.
 
 =cut
 
-method text { $_text }
+# generated accessor
 
 =head2 set_text
 
