@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
 
-use Object::Pad 0.27;
+use Object::Pad 0.57;
 
-package Tickit::Widget::Frame 0.33;
+package Tickit::Widget::Frame 0.36;
 class Tickit::Widget::Frame
-   extends Tickit::SingleChildWidget;
+   :isa(Tickit::SingleChildWidget);
 
 use Tickit::Style;
 
@@ -131,20 +131,21 @@ For more details see the accessors below.
 
 =cut
 
-has $_title;
+has $_title    :reader :param = undef;
 has %_has_edge;
 
-BUILD
+ADJUSTPARAMS
 {
-   my %args = @_;
+   my ( $params ) = @_;
 
-   $self->set_title( $args{title} ) if defined $args{title};
-   $self->set_title_align( $args{title_align} || 0 );
+   $self->set_title_align( delete $params->{title_align} || 0 );
 
    # Prepopulate has_* caches
    $self->on_style_changed_values;
 
-   $self->set_child( $args{child} ) if $args{child};
+   if( exists $params->{child} ) {
+      croak "The 'child' constructor argument to ${\ref $self} is no longer recognised; use ->add_child instead";
+   }
 }
 
 =head1 ACCESSORS
@@ -216,10 +217,7 @@ my %LINESTYLES = (
 
 =cut
 
-method title
-{
-   return $_title;
-}
+# generated accessor
 
 =head2 set_title
 
