@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 
-use Object::Pad 0.27;
+use Object::Pad 0.57;
 
-package Tickit::Widget::GridBox 0.31;
+package Tickit::Widget::GridBox 0.34;
 class Tickit::Widget::GridBox
-   extends Tickit::ContainerWidget;
+   :isa(Tickit::ContainerWidget);
 
 use experimental 'postderef';
 
@@ -87,20 +87,6 @@ use constant WIDGET_PEN_FROM_STYLE => 1;
 
 Constructs a new C<Tickit::Widget::GridBox> object.
 
-Takes the following named arguments:
-
-=over 8
-
-=item children => ARRAY[ARRAY[Tickit::Widget]]
-
-Optional. If present, should be a 2D ARRAYref of ARRAYrefs containing the
-C<Tickit::Widget> children to display in the grid. They are all added with no
-additional options.
-
-This is now discouraged in favour of the L</append_row> method.
-
-=back
-
 =cut
 
 sub BUILDARGS {
@@ -115,18 +101,12 @@ sub BUILDARGS {
 has @_grid;
 has $_max_col = -1;
 
-BUILD
+ADJUSTPARAMS
 {
-   my %args = @_;
+   my ( $params ) = @_;
 
-   if( my $children = $args{children} ) {
-      Carp::carp( "The 'children' constructor argument to ${\ref $self} is discouraged; use ->append_row instead" );
-
-      foreach my $row ( 0 .. $#$children ) {
-         foreach my $col ( 0 .. $#{ $children->[$row] } ) {
-            $self->add( $row, $col, $children->[$row][$col] );
-         }
-      }
+   if( $params->{children} ) {
+      croak "The 'children' constructor argument to ${\ref $self} is no longer recognised; use ->append_row instead";
    }
 }
 
@@ -414,9 +394,6 @@ suitable to use as a chaining mutator; e.g.
                       Tickit::Widget::Static->new( ... ) ] )
       ->append_row( ... );
 
-This should be preferred over using the C<children> constructor argument,
-which is now discouraged.
-
 =cut
 
 method append_col
