@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2021 -- leonerd@leonerd.org.uk
 
-use Object::Pad 0.27;
+use Object::Pad 0.57;
 
-package Tickit::Widget::Spinner 0.31;
+package Tickit::Widget::Spinner 0.34;
 class Tickit::Widget::Spinner
-   extends Tickit::Widget;
+   :isa(Tickit::Widget);
 
 use experimental 'postderef';
 
@@ -72,16 +72,14 @@ Optional. The time each string is displayed for. Defaults to 0.5.
 
 has @_chars;
 has $_state = 0;
-has $_interval;
+has $_interval :param = 0.5;
 has $_cols;
 
-BUILD
+ADJUSTPARAMS
 {
-   my %params = @_;
+   my ( $params ) = @_;
 
-   @_chars = $params{chars} ? $params{chars}->@* : (qw( - \ | / ));
-
-   $_interval = $params{interval} // 0.5;
+   @_chars = $params->{chars} ? $params->{chars}->@* : (qw( - \ | / ));
 
    $_cols = max map { textwidth $_ } @_chars;
 }
