@@ -1,13 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2021 -- leonerd@leonerd.org.uk
 
-use Object::Pad 0.27;
+use Object::Pad 0.57;
 
-package Tickit::Widget::Static 0.52;
+package Tickit::Widget::Static 0.55;
 class Tickit::Widget::Static
-   extends Tickit::Widget;
+   :isa(Tickit::Widget);
 
 use Tickit::Style;
 
@@ -90,19 +90,15 @@ For more details see the accessors below.
 =cut
 
 has @_lines;
-has $_on_click;
+has $_on_click :reader :writer :param = undef;
 
-BUILD
+ADJUSTPARAMS
 {
-   my %params = @_;
+   my ( $params ) = @_;
 
-   $self->set_text( $params{text} );
-   $self->set_align( $params{align} || 0 );
-   $self->set_valign( $params{valign} || 0 );
-
-   $self->set_on_click( $params{on_click} );
-
-   return $self;
+   $self->set_text( delete $params->{text} );
+   $self->set_align( delete $params->{align} || 0 );
+   $self->set_valign( delete $params->{valign} || 0 );
 }
 
 =head1 ACCESSORS
@@ -173,10 +169,7 @@ See also L<Tickit::WidgetRole::Alignable>.
 
 =cut
 
-method set_on_click
-{
-   ( $_on_click ) = @_;
-}
+# generated accessor
 
 method render_to_rb
 {
